#![allow(clippy::expect_used)]
use askama::Template;
use axum::{
    Router,
    extract::{Path, State},
    http::HeaderMap,
    http::StatusCode,
    response::{Html, IntoResponse},
    routing::get,
    serve,
};
use person::{PersonID, PersonTemplate, PersonTemplateFull};
use stammbaum::{Stammbaum, StammbaumFull};
use std::io::BufReader;
use tokio::runtime::Runtime;
use tower_http::trace::TraceLayer;
use tracing::{error, instrument, warn};

mod person;
mod stammbaum;
#[instrument(skip(stammbaum))]
async fn get_person(
    Path(id): Path<String>,
    headers: HeaderMap,
    State(stammbaum): State<Stammbaum>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    let id = match PersonID::try_from(id) {
        Ok(id) => id,
        Err(e) => {
            error!(error = ?e);
            return Err(StatusCode::INTERNAL_SERVER_ERROR);
        }
    };
    match stammbaum.get(id) {
        None => {
            warn!(id = ?id, "not_found");
            Err(StatusCode::NOT_FOUND)
        }
        Some(p) => match headers.get("HX-Request") {
            Some(_) => Ok(Html(PersonTemplate::from(p.to_owned()).render().map_err(
                |e| {
                    error!(error=?e, id = ?id);
                    StatusCode::INTERNAL_SERVER_ERROR
                },
            )?)),
            _ => Ok(Html(
                PersonTemplateFull::from(p.to_owned())
                    .render()
                    .map_err(|e| {
                        error!(error=?e, id = ?id);
                        StatusCode::INTERNAL_SERVER_ERROR
                    })?,
            )),
        },
    }
}

#[instrument]
async fn list_stammbaum(
    headers: HeaderMap,
    State(stammbaum): State<Stammbaum>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    match headers.get("HX-Request") {
        Some(_) => Ok::<axum::response::Html<std::string::String>, StatusCode>(Html(
            stammbaum
                .render()
                .expect("should have rendered the template"),
        )),
        None => Ok::<axum::response::Html<std::string::String>, StatusCode>(Html(
            StammbaumFull::from(stammbaum)
                .render()
                .expect("should have rendered the template"),
        )),
    }
}

pub fn run() -> Result<(), String> {
    let rt = Runtime::new().map_err(|e| e.to_string())?;
    rt.block_on(async {
        #[allow(clippy::expect_used)]
        let stammbaum: Stammbaum = serde_json::from_reader(BufReader::new(
            std::fs::File::open("test.json").expect("tbd file"),
        ))
        .expect("tbd struct");

        let app = Router::new()
            .route("/", get(list_stammbaum))
            .route("/{id}", get(get_person))
            .layer(TraceLayer::new_for_http())
            .with_state(stammbaum);

        let addr = tokio::net::TcpListener::bind("0.0.0.0:3000")
            .await
            .expect("faild to bind port");

        serve(addr, app).await.map_err(|e| e.to_string())
    })
    .map_err(|e| e.to_string())?;

    Ok(())
}
