use std::sync::Arc;

use askama::Template;
use chrono::{DateTime, Utc};
use serde::{Deserialize, Serialize};
use uuid::Uuid;

pub type PersonID = Uuid;

#[derive(Template)]
#[template(path = "person.html")]
pub struct PersonTemplate {
    person: Person,
}

impl From<Person> for PersonTemplate {
    fn from(value: Person) -> Self {
        Self { person: value }
    }
}
#[derive(Template)]
#[template(path = "person_full.html")]
pub struct PersonTemplateFull {
    person: Person,
}

impl From<Person> for PersonTemplateFull {
    fn from(value: Person) -> Self {
        Self { person: value }
    }
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Person {
    id: PersonID,
    first_name: Arc<str>,
    last_name: Arc<str>,
    #[serde(skip_serializing_if = "Option::is_none")]
    maiden_name: Option<Arc<str>>,
    sex: Sex,
    date_of_birth: DateTime<Utc>,
    parents: Vec<PersonID>,
    marriages: Vec<Marriage>,
}

#[allow(dead_code)]
impl Person {
    pub fn new<T: AsRef<str>>(
        first_name: T,
        last_name: T,
        maiden_name: Option<T>,
        sex: Sex,
        date_of_birth: DateTime<Utc>,
        parents: Vec<PersonID>,
    ) -> Self {
        let uuid = Uuid::new_v4();
        Self {
            id: uuid,
            first_name: Arc::from(first_name.as_ref()),
            last_name: Arc::from(last_name.as_ref()),
            maiden_name: maiden_name.map(|m| Arc::from(m.as_ref())),
            date_of_birth,
            sex,
            parents,
            marriages: Vec::with_capacity(1),
        }
    }

    pub fn sex(&self) -> &Sex {
        &self.sex
    }

    pub fn date_of_birth(&self) -> DateTime<Utc> {
        self.date_of_birth
    }

    pub fn maiden_name(&self) -> Option<Arc<str>> {
        self.maiden_name.clone()
    }

    pub fn last_name(&self) -> Arc<str> {
        self.last_name.clone()
    }
    pub fn first_name(&self) -> Arc<str> {
        self.first_name.clone()
    }

    pub fn parents(&self) -> &Vec<PersonID> {
        &self.parents
    }

    pub fn add_parent(&mut self, parent: PersonID) {
        self.parents.push(parent);
    }

    pub fn add_marriage(&mut self, marriage: Marriage) {
        self.marriages.push(marriage);
    }

    pub fn id(&self) -> Uuid {
        self.id
    }
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub enum Sex {
    Male,
    Female,
    Other(Arc<str>),
}

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct Marriage {
    wife: PersonID,
    husband: PersonID,
    date: DateTime<Utc>,
    divorce_date: Option<DateTime<Utc>>,
}

#[allow(dead_code)]
impl Marriage {
    pub fn new(
        wife: PersonID,
        husband: PersonID,
        date: DateTime<Utc>,
        divorce_date: Option<DateTime<Utc>>,
    ) -> Self {
        Self {
            wife,
            husband,
            date,
            divorce_date,
        }
    }

    pub fn divorce(&mut self, date: DateTime<Utc>) {
        self.divorce_date = Some(date);
    }
}
